
#include <LovyanGFX.hpp>
#include <lvgl.h>
#include <demos/lv_demos.h>
#include <Arduino.h>
//  Create a class that does your own settings, derived from LGFX_Device.
class LGFX : public lgfx::LGFX_Device
{
    lgfx::Panel_ILI9341      _panel_instance;
    lgfx::Bus_SPI            _bus_instance;
    lgfx::Touch_XPT2046      _touch_instance;

  public:

    LGFX(void)
    {
      { //  Configure bus control settings.
        auto cfg = _bus_instance.config();  //  Get the structure for the bus configuration.

        //  SPI bus configuration
        cfg.spi_host    = VSPI_HOST;        //  Select the SPI to use ESP32-S2,C3 : SPI2_HOST or SPI3_HOST / ESP32 : VSPI_HOST or HSPI_HOST
        //  * Due to the ESP-IDF version upgrade, VSPI_HOST and HSPI_HOST descriptions are deprecated, so if an error occurs, use SPI2_HOST and SPI3_HOST instead.
        cfg.spi_mode    = 0;                //  Set SPI communication mode (0 ~ 3)
        cfg.freq_write  = 40000000;         //  80000000 // SPI clock frequency when sending (max 80MHz, rounded to 80MHz divided by an integer)
        cfg.freq_read   = 16000000;         //  SPI clock frequency when receiving
        cfg.spi_3wire   = true;             //  Set true if receiving on the MOSI
        cfg.use_lock    = true;             //  set true to use transaction
        cfg.dma_channel = SPI_DMA_CH_AUTO;  //  Set the DMA channel to use (0=not use DMA / 1=1ch / 2=ch / SPI_DMA_CH_AUTO=auto setting)
        //  * With the ESP-IDF version upgrade, SPI_DMA_CH_AUTO (automatic setting) is recommended for the DMA channel.   Specifying 1ch and 2ch is deprecated.
        cfg.pin_sclk    = 14;               //  SPI SCLK
        cfg.pin_mosi    = 13;               //  SPI MOSI
        cfg.pin_miso    = 33;               //  SPI MISO      (-1 = disable)
        cfg.pin_dc      =  2;               //  SPI D/C = A0  (-1 = disable)
        //  When using the same SPI bus as the SD card, be sure to set MISO without omitting it.

        _bus_instance.config(cfg);
        _panel_instance.setBus(&_bus_instance);
      }

      { // Set display panel controls.
        auto cfg = _panel_instance.config();

        cfg.pin_cs           =    15;  // CS    (-1 = disable)
        cfg.pin_rst          =    -1;  // RST   (-1 = disable)
        cfg.pin_busy         =    -1;  // BUSY  (-1 = disable)

        // * The following setting values ​​are general initial values ​​for each panel, so please try commenting out any unknown items.

        cfg.panel_width      =   240;  // actual displayable width
        cfg.panel_height     =   320;  // actual displayable  height
        cfg.offset_x         =     0;  // Panel offset in X
        cfg.offset_y         =     0;  // Panel offset in Y
        cfg.offset_rotation  =     2;  // Rotation value offset 0~7 (4~7 are upside down
        cfg.dummy_read_pixel =     8;  // Number of dummy read bits before pixel read
        cfg.dummy_read_bits  =     1;  // Number of dummy read bits before non-pixel data read
        cfg.readable         =  true;  // Data can be read set to true
        cfg.invert           = false;  // if panel light and dark are inverted set to true
        cfg.rgb_order        = false;  // if panel red and blue are reversed set to true
        cfg.dlen_16bit       = false;  // Set to true for panels that transmit data length in 16-bit units with 16-bit parallel or SPI
        cfg.bus_shared       =  true;  // If the bus is shared with the SD card, set to true (bus control with drawJpgFile etc.)

        // Please set below only with drivers that can change the number of pixels such as ST7735 and ILI9163.
        // Please set the following only when the display shifts with a driver with a variable number of pixels such as ST7735 or ILI9163.
        //cfg.memory_width     =   240;  // Maximum width  supported by the driver IC
        //cfg.memory_height    =   320;  // Maximum height supported by the driver IC

        _panel_instance.config(cfg);
      }

      { // Configure settings for touch screen control.  (delete if not necessary)

        auto cfg = _touch_instance.config();

        // numbers for x_min etc. doesn't work
        cfg.x_min           =  340;  // Minimum X value (raw value) obtained from the touchscreen
        cfg.x_max           = 3900;  // Maximum X value (raw value) obtained from the touchscreen
        cfg.y_min           =  235;  // Minimum Y value obtained from touchscreen (raw value)
        cfg.y_max           = 3900;  // Maximum Y value (raw value) obtained from touchscreen
        cfg.pin_int         =   -1;  // Pin number to which INT is connected (-1 = not connected)
        cfg.bus_shared      = true;  // Set true when using a common bus with the screen
        cfg.offset_rotation =    2;  // Adjustment when the orientation does not match Set with a value from 0 to 7
        //cfg.offset_rotation =  6;  // 6 fits but double lines. why?

        // For SPI connection
        cfg.spi_host = VSPI_HOST;    // Select SPI to use (HSPI_HOST or VSPI_HOST)
        cfg.freq     = 2500000;      // SPI Clock frequency 1000000 -> 2500000
        cfg.pin_sclk = 14;           // SCLK
        cfg.pin_mosi = 13;           // MOSI
        cfg.pin_miso = 33;           // MISO
        cfg.pin_cs   =  12;           // CS

        _touch_instance.config(cfg);
        _panel_instance.setTouch(&_touch_instance);
      }
      setPanel(&_panel_instance);
    }
};

//  Create an instance of the prepared class.
LGFX display;
//  Modified version of : Arduino\libraries\LovyanGFX\examples\Lovyan_user_setting.ino
//  Also modified is    : Arduino\libraries\LovyanGFX\src\lgfx\v1\LGFXBase.cpp ( LGFX_Device::calibrate_touch )
//                        Serial monitor shows now the calibration data, if you call display.setTouchCalibrate(calData);
/*更改屏幕分辨率*/
static const uint16_t screenWidth  = 320;
static const uint16_t screenHeight = 240;

static lv_disp_draw_buf_t draw_buf;
static lv_color_t buf1[ screenWidth * screenHeight / 10 ];
/* 显示器刷新 */
void my_disp_flush( lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p )
{
  uint32_t w = ( area->x2 - area->x1 + 1 );
  uint32_t h = ( area->y2 - area->y1 + 1 );

  display.startWrite();
  display.setAddrWindow( area->x1, area->y1, w, h );
  display.pushColors( ( uint16_t * )&color_p->full, w * h, true );
  display.endWrite();

  lv_disp_flush_ready( disp );
}

/*Read the touchpad*/
void my_touchpad_read( lv_indev_drv_t * indev_driver, lv_indev_data_t * data )
{
  uint16_t touchX, touchY;
  bool touched = display.getTouch( &touchX, &touchY);
  if ( !touched )
  {
    data->state = LV_INDEV_STATE_REL;
  }
  else
  {
    data->state = LV_INDEV_STATE_PR;

    /*Set the coordinates*/
    data->point.x = touchX;
    data->point.y = touchY;

    Serial.print( "Data x " );
    Serial.println( touchX );

    Serial.print( "Data y " );
    Serial.println( touchY );
  }
}
void setup(void)
{
  pinMode(27, OUTPUT);
  digitalWrite(27, HIGH);
  Serial.begin(115200);
  // Enabled after SPI bus and panel initialization.
  display.begin();
  display.setRotation(1); /* 旋转 */
  display.setBrightness(255);
  uint16_t calData[8] ={3806, 221, 3804, 3635, 425, 199, 394, 3655,}; //Touch correction requires modifying data according to the actual situation of the screen
  display.setTouchCalibrate(calData);
  //  display.setTextSize((std::max(display.width(), display.height()) + 255) >> 8);
  //
  //  display.setRotation(0);
  //  display.drawString("Top left", 10, 10);
  //
  //  // Calibrate if touch is enabled.  (Optional)
  //  if (display.touch())
  //  {
  //    if (display.width() < display.height()) display.setRotation(display.getRotation() ^ 1);
  //    // Draw the guidance text on the screen.
  //    display.setTextDatum(textdatum_t::middle_center);
  //    display.drawString("touch the arrow.", display.width()>>1, display.height() >> 1);
  //    display.setTextDatum(textdatum_t::top_left);
  //
  //    // If using touch, calibrate. Touch the tips of the arrows displayed in the four corners of the screen in order.
  //    std::uint16_t fg = TFT_WHITE;
  //    std::uint16_t bg = TFT_BLACK;
  //    if (display.isEPD()) std::swap(fg, bg);
  //display.calibrateTouch(nullptr, fg, bg, std::max(display.width(), display.height()) >> 3);

  // Calibration data with modified Arduino\libraries\LovyanGFX\src\lgfx\v1\LGFXBase.cpp
  // Serial monitor shows now calibration data
  //...
  //panel()->setCalibrate(orig);
  //setRotation(rot);
  //Serial.print("Calibration data : ");
  //for (int j=0; j < 8;  j++) {
  //  Serial.print(orig[j]);
  //  Serial.print(", ");
  //}
  //Serial.println("done");
  //...

  //Calibration data for offset_rotation of the panel = 2 and offset_rotation of the touch = 0 (?)
  //Calibration data : 3874, 365, 3823, 3882, 328, 361, 275, 3898, done
  //Calibration data : 3863, 376, 3837, 3922, 293, 378, 247, 3930, done
  //Calibration data : 3874, 385, 3870, 3948, 256, 357, 223, 3913, done
  //Calibration data : 3896, 350, 3868, 3957, 280, 354, 216, 3925, done
  //  }

  //  uint16_t calData[8] = {3890, 340, 3900, 3895, 235, 340, 235, 3900};
  //  display.setTouchCalibrate(calData);

   display.fillScreen(TFT_BLACK);
  //lvgl初始化
  lv_init();
  lv_disp_draw_buf_init( &draw_buf, buf1, NULL, screenWidth * screenHeight / 10 );

  /*初始化显示*/
  static lv_disp_drv_t disp_drv;
  lv_disp_drv_init( &disp_drv );
  /*将以下行更改为显示分辨率*/
  disp_drv.hor_res = screenWidth;
  disp_drv.ver_res = screenHeight;
  disp_drv.flush_cb = my_disp_flush;
  disp_drv.draw_buf = &draw_buf;
  lv_disp_drv_register( &disp_drv );

  /*Initialize the (dummy) input device driver*/
  static lv_indev_drv_t indev_drv;
  lv_indev_drv_init(&indev_drv);
  indev_drv.type = LV_INDEV_TYPE_POINTER;
  indev_drv.read_cb = my_touchpad_read;
  lv_indev_drv_register(&indev_drv);

  lv_demo_widgets();//主UI界面
}

uint32_t count = ~0;
void loop(void)
{


  //  display.fillScreen(TFT_RED);
  //  delay(1000);
  //  display.fillScreen(TFT_GREEN);
  //  delay(1000);
  //  display.fillScreen(TFT_BLUE);
  //  delay(1000);
  //  display.fillScreen(TFT_BLACK);
  delay(10);

  lv_timer_handler();

}
//void setup(void)
//{
//  pinMode(27, OUTPUT);
//  digitalWrite(27, HIGH);
//  Serial.begin(115200);
//  // Enabled after SPI bus and panel initialization.
//  display.init();
//
//  display.setTextSize((std::max(display.width(), display.height()) + 255) >> 8);
//
//  display.setRotation(0);
//  display.drawString("Top left", 10, 10);
//  uint16_t calData[8] = {3806, 221, 3804, 3635, 425, 199, 394, 3655,};
//  // Calibrate if touch is enabled.  (Optional)
//  if (display.touch())
//  {
//    if (display.width() < display.height()) display.setRotation(display.getRotation() ^ 1);
//    // Draw the guidance text on the screen.
//    display.setTextDatum(textdatum_t::middle_center);
//    display.drawString("touch the arrow.", display.width() >> 1, display.height() >> 1);
//    display.setTextDatum(textdatum_t::top_left);
//
//    // If using touch, calibrate. Touch the tips of the arrows displayed in the four corners of the screen in order.
//    std::uint16_t fg = TFT_WHITE;
//    std::uint16_t bg = TFT_BLACK;
//    if (display.isEPD()) std::swap(fg, bg);
//    display.calibrateTouch(calData, fg, bg, std::max(display.width(), display.height()) >> 3);
//
//    // Calibration data with modified Arduino\libraries\LovyanGFX\src\lgfx\v1\LGFXBase.cpp
//    // Serial monitor shows now calibration data
//    //...
////    panel()->setCalibrate(orig);
////    setRotation(rot);
//    Serial.print("Calibration data : ");
//    for (int j=0; j < 8;  j++) {
//      Serial.print(calData[j]);
//      Serial.print(", ");
//    }
//    Serial.println("done");
//    //...
//
//    //Calibration data for offset_rotation of the panel = 2 and offset_rotation of the touch = 0 (?)
//    //Calibration data : 3874, 365, 3823, 3882, 328, 361, 275, 3898, done
//    //Calibration data : 3863, 376, 3837, 3922, 293, 378, 247, 3930, done
//    //Calibration data : 3874, 385, 3870, 3948, 256, 357, 223, 3913, done
//    //Calibration data : 3896, 350, 3868, 3957, 280, 354, 216, 3925, done
//  }
//
//  
//  display.setTouchCalibrate(calData);
//
//  display.fillScreen(TFT_BLACK);
//}
//
//uint32_t count = ~0;
//void loop(void)
//{
//  display.startWrite();
//  display.setRotation(++count & 7);
//  //display.setColorDepth((count & 8) ? 16 : 24);
//  //display.setColorDepth( 16 );
//
//  display.setTextColor(TFT_WHITE);
//  display.drawNumber(display.getRotation(), 16, 2);
//
//  display.setTextColor(0xFF0000U);
//  display.drawString("R", 30, 16);
//  display.setTextColor(0x00FF00U);
//  display.drawString("G", 40, 16);
//  display.setTextColor(0x0000FFU);
//  display.drawString("B", 50, 16);
//
//  display.drawRect(30, 30, display.width() - 60, display.height() - 60, 0x00FF00U);
//  display.drawFastHLine(0, 0, 10);
//
//  display.endWrite();
//
//  int32_t x, y;
//  if (display.getTouch(&x, &y)) {
//    display.fillRect(x, y, 1, 1, 0xFFFFFFU);
//  }
//}
